'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createAction;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _constants = require('./constants');

var id = 0;

var types = {};

var identity = function identity(arg) {
  return arg;
};

var undef = function undef() {
  return undefined;
};

function createAction(description, payloadReducer, metaReducer) {
  if (typeof description === 'function') {
    metaReducer = payloadReducer;
    payloadReducer = description;
    description = undefined;
  }

  if (typeof payloadReducer !== 'function') {
    payloadReducer = identity;
  }

  if (typeof metaReducer !== 'function') {
    metaReducer = undef;
  }

  var isSerializable = typeof description === 'string' && /^[A-Z_]+$/.test(description);

  if (isSerializable) {
    if (types[description]) {
      throw new TypeError('Duplicate action type: ' + description);
    }

    types[description] = true;
  }

  var action = {
    id: isSerializable ? description : ++id,
    type: isSerializable ? description : '[' + id + ']' + (description ? ' ' + description : '')
  };

  var actionStores = undefined;

  function actionCreator() {
    var _payloaded;

    var payloaded = (_payloaded = {}, _defineProperty(_payloaded, _constants.ID, action.id), _defineProperty(_payloaded, 'type', action.type), _defineProperty(_payloaded, 'payload', payloadReducer.apply(undefined, arguments)), _defineProperty(_payloaded, 'meta', metaReducer.apply(undefined, arguments)), _payloaded);

    if (Array.isArray(actionStores)) {
      return actionStores.map(function (store) {
        return store.dispatch(payloaded);
      });
    } else if (actionStores) {
      return actionStores.dispatch(payloaded);
    } else {
      return payloaded;
    }
  }

  actionCreator.toString = function () {
    return action.id;
  };

  actionCreator.bindTo = function (stores) {
    actionStores = stores;
    return actionCreator;
  };

  return actionCreator;
}

;
module.exports = exports['default'];